@@ -81,6 +81,10 @@ class Device(ABC, Generic[T]):
     def load_filter(self, slot: str, entry: CatalogueEntry) -> None:
         pass
 
+    @abstractmethod
+    def load_biquads(self, slot: str, overwrite: bool, inputs: List[int], outputs: List[int], biquads: List[dict]) -> None:
+        pass
+
     @abstractmethod
     def clear_filter(self, slot: str) -> None:
         pass
@@ -137,6 +141,10 @@ class DeviceRepository:
     def load_filter(self, name: str, slot: str, entry: CatalogueEntry) -> None:
         self.__get_device(name).load_filter(slot, entry)
 
+    def load_biquads(self, name: str, slot: str, overwrite: bool, inputs: List[int], outputs: List[int],
+                     biquads: List[dict]) -> None:
+        self.__get_device(name).load_biquads(slot, overwrite, inputs, outputs, biquads)
+
     def clear_filter(self, name: str, slot: str) -> None:
         self.__get_device(name).clear_filter(slot)
 
@@ -169,6 +177,9 @@ def create_devices(cfg: Config, ws_server: WsServer, catalogue: CatalogueProvide
         elif d_type == 'jriver':
             from ezbeq.jriver import JRiver
             devices.append(JRiver(name, cfg.config_path, values, ws_server, catalogue))
+        elif d_type == 'qsys':
+            from ezbeq.qsys import Qsys
+            devices.append(Qsys(name, cfg.config_path, values, ws_server, catalogue))
     if not devices:
         raise ValueError('No device configured')
     else:
